@@ -56,7 +56,7 @@ func decodeMap(d *Decoder) (interface{}, error) {
 		return nil, nil
 	}
 
-	m := make(map[interface{}]interface{}, n)
+	m := make(map[interface{}]interface{}, min(n, mapAllocLimit))
 	for i := 0; i < n; i++ {
 		mk, err := d.DecodeInterface()
 		if err != nil {
@@ -76,6 +76,12 @@ func (d *Decoder) DecodeMapLen() (int, error) {
 	if err != nil {
 		return 0, err
 	}
+	if codes.IsExt(c) {
+		c, err = d.skipExtHeader(c)
+		if err != nil {
+			return 0, err
+		}
+	}
 	return d.mapLen(c)
 }
 
@@ -114,7 +120,7 @@ func (d *Decoder) decodeMapStringStringPtr(ptr *map[string]string) error {
 
 	m := *ptr
 	if m == nil {
-		*ptr = make(map[string]string, n)
+		*ptr = make(map[string]string, min(n, mapAllocLimit))
 		m = *ptr
 	}
 
@@ -150,7 +156,7 @@ func (d *Decoder) decodeMapStringInterfacePtr(ptr *map[string]interface{}) error
 
 	m := *ptr
 	if m == nil {
-		*ptr = make(map[string]interface{}, n)
+		*ptr = make(map[string]interface{}, min(n, mapAllocLimit))
 		m = *ptr
 	}
 
