@@ -19,36 +19,32 @@ type byteWriter struct {
 	io.Writer
 }
 
-func (w *byteWriter) WriteByte(b byte) error {
-	n, err := w.Write([]byte{b})
-	if err != nil {
-		return err
-	}
-	if n != 1 {
-		return io.ErrShortWrite
-	}
-	return nil
+func (w byteWriter) WriteByte(b byte) error {
+	_, err := w.Write([]byte{b})
+	return err
 }
 
-func (w *byteWriter) WriteString(s string) (int, error) {
+func (w byteWriter) WriteString(s string) (int, error) {
 	return w.Write([]byte(s))
 }
 
 func Marshal(v ...interface{}) ([]byte, error) {
-	buf := &bytes.Buffer{}
-	err := NewEncoder(buf).Encode(v...)
+	var buf bytes.Buffer
+	err := NewEncoder(&buf).Encode(v...)
 	return buf.Bytes(), err
 }
 
 type Encoder struct {
 	w   writer
 	buf []byte
+
+	sortMapKeys bool
 }
 
 func NewEncoder(w io.Writer) *Encoder {
 	bw, ok := w.(writer)
 	if !ok {
-		bw = &byteWriter{Writer: w}
+		bw = byteWriter{Writer: w}
 	}
 	return &Encoder{
 		w:   bw,
@@ -56,6 +52,15 @@ func NewEncoder(w io.Writer) *Encoder {
 	}
 }
 
+// SortMapKeys causes the Encoder to encode map keys in increasing order.
+// Supported map types are:
+//   - map[string]string
+//   - map[string]interface{}
+func (e *Encoder) SortMapKeys(v bool) *Encoder {
+	e.sortMapKeys = v
+	return e
+}
+
 func (e *Encoder) Encode(v ...interface{}) error {
 	for _, vv := range v {
 		if err := e.encode(vv); err != nil {
@@ -89,8 +94,6 @@ func (e *Encoder) encode(v interface{}) error {
 		return e.EncodeFloat64(v)
 	case []string:
 		return e.encodeStringSlice(v)
-	case map[string]string:
-		return e.encodeMapStringString(v)
 	case time.Duration:
 		return e.EncodeInt64(int64(v))
 	case time.Time:
@@ -102,6 +105,8 @@ func (e *Encoder) encode(v interface{}) error {
 		}
 		_, err = e.w.Write(b)
 		return err
+	case CustomEncoder:
+		return v.EncodeMsgpack(e)
 	}
 	return e.EncodeValue(reflect.ValueOf(v))
 }
