@@ -8,31 +8,32 @@ import (
 )
 
 func ExampleMarshal() {
-	b, err := msgpack.Marshal(true)
+	type Item struct {
+		Foo string
+	}
+
+	b, err := msgpack.Marshal(&Item{Foo: "bar"})
 	if err != nil {
 		panic(err)
 	}
-	fmt.Printf("%#v\n", b)
-	// Output:
 
-	var out bool
-	err = msgpack.Unmarshal([]byte{0xc3}, &out)
+	var item Item
+	err = msgpack.Unmarshal(b, &item)
 	if err != nil {
 		panic(err)
 	}
-	fmt.Println(out)
-	// Output: []byte{0xc3}
-	// true
-}
-
-type myStruct struct {
-	S string
+	fmt.Println(item.Foo)
+	// Output: bar
 }
 
 func ExampleRegisterExt() {
-	msgpack.RegisterExt(1, myStruct{})
+	type Item struct {
+		S string
+	}
 
-	b, err := msgpack.Marshal(&myStruct{S: "string"})
+	msgpack.RegisterExt(1, Item{})
+
+	b, err := msgpack.Marshal(&Item{S: "string"})
 	if err != nil {
 		panic(err)
 	}
@@ -43,7 +44,7 @@ func ExampleRegisterExt() {
 		panic(err)
 	}
 	fmt.Printf("%#v", v)
-	// Output: msgpack_test.myStruct{S:"string"}
+	// Output: msgpack_test.Item{S:"string"}
 }
 
 func Example_mapStringInterface() {
@@ -68,7 +69,7 @@ func Example_mapStringInterface() {
 }
 
 func Example_recursiveMapStringInterface() {
-	buf := &bytes.Buffer{}
+	buf := new(bytes.Buffer)
 
 	enc := msgpack.NewEncoder(buf)
 	in := map[string]interface{}{"foo": map[string]interface{}{"hello": "world"}}
@@ -101,3 +102,29 @@ func Example_recursiveMapStringInterface() {
 	fmt.Printf("%v %#v\n", err, out)
 	// Output: <nil> map[string]interface {}{"foo":map[string]interface {}{"hello":"world"}}
 }
+
+func ExampleDecoder_Query() {
+	b, err := msgpack.Marshal([]map[string]interface{}{
+		{"id": 1, "attrs": map[string]interface{}{"phone": 12345}},
+		{"id": 2, "attrs": map[string]interface{}{"phone": 54321}},
+	})
+	if err != nil {
+		panic(err)
+	}
+
+	dec := msgpack.NewDecoder(bytes.NewBuffer(b))
+	values, err := dec.Query("*.attrs.phone")
+	if err != nil {
+		panic(err)
+	}
+	fmt.Println("phones are", values)
+
+	dec.Reset(bytes.NewBuffer(b))
+	values, err = dec.Query("1.attrs.phone")
+	if err != nil {
+		panic(err)
+	}
+	fmt.Println("2nd phone is", values[0])
+	// Output: phones are [12345 54321]
+	// 2nd phone is 54321
+}
